@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,20 +22,19 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
 
 #include "cppeditorplugin.h"
 
-#include "cppclasswizard.h"
+#include "cppautocompleter.h"
 #include "cppcodemodelinspectordialog.h"
 #include "cppeditorconstants.h"
 #include "cppeditor.h"
-#include "cppeditoroutline.h"
-#include "cppfilewizard.h"
+#include "cppeditordocument.h"
 #include "cpphighlighter.h"
 #include "cpphoverhandler.h"
 #include "cppincludehierarchy.h"
@@ -45,8 +44,15 @@
 #include "cppsnippetprovider.h"
 #include "cpptypehierarchy.h"
 
+#include <coreplugin/editormanager/editormanager.h>
+
+#ifdef WITH_TESTS
+#  include "cppdoxygen_test.h"
+#endif
+
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/actionmanager.h>
+#include <coreplugin/actionmanager/command.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/fileiconprovider.h>
 #include <coreplugin/icore.h>
@@ -55,17 +61,20 @@
 #include <cpptools/cpptoolsconstants.h>
 #include <texteditor/texteditoractionhandler.h>
 #include <texteditor/texteditorconstants.h>
-#include <texteditor/texteditorsettings.h>
-#include <texteditor/highlighterfactory.h>
 
 #include <utils/hostosinfo.h>
+#include <utils/mimetypes/mimedatabase.h>
+#include <utils/theme/theme.h>
 
 #include <QCoreApplication>
 #include <QStringList>
 
 using namespace Core;
-using namespace CppEditor;
-using namespace CppEditor::Internal;
+using namespace TextEditor;
+using namespace Utils;
+
+namespace CppEditor {
+namespace Internal {
 
 void registerQuickFixes(ExtensionSystem::IPlugin *plugIn);
 
@@ -73,43 +82,53 @@ enum { QUICKFIX_INTERVAL = 20 };
 
 //////////////////////////// CppEditorFactory /////////////////////////////
 
-CppEditorFactory::CppEditorFactory(CppEditorPlugin *owner) :
-    m_owner(owner)
+class CppEditorFactory : public TextEditorFactory
 {
-    setId(CppEditor::Constants::CPPEDITOR_ID);
-    setDisplayName(qApp->translate("OpenWith::Editors", CppEditor::Constants::CPPEDITOR_DISPLAY_NAME));
-    addMimeType(CppEditor::Constants::C_SOURCE_MIMETYPE);
-    addMimeType(CppEditor::Constants::C_HEADER_MIMETYPE);
-    addMimeType(CppEditor::Constants::CPP_SOURCE_MIMETYPE);
-    addMimeType(CppEditor::Constants::CPP_HEADER_MIMETYPE);
+public:
+    CppEditorFactory()
+    {
+        setId(Constants::CPPEDITOR_ID);
+        setDisplayName(qApp->translate("OpenWith::Editors", Constants::CPPEDITOR_DISPLAY_NAME));
+        addMimeType(Constants::C_SOURCE_MIMETYPE);
+        addMimeType(Constants::C_HEADER_MIMETYPE);
+        addMimeType(Constants::CPP_SOURCE_MIMETYPE);
+        addMimeType(Constants::CPP_HEADER_MIMETYPE);
 
-    new TextEditor::TextEditorActionHandler(this, CppEditor::Constants::C_CPPEDITOR,
-        TextEditor::TextEditorActionHandler::Format
-        | TextEditor::TextEditorActionHandler::UnCommentSelection
-        | TextEditor::TextEditorActionHandler::UnCollapseAll
-        | TextEditor::TextEditorActionHandler::FollowSymbolUnderCursor);
+        setDocumentCreator([]() { return new CppEditorDocument; });
+        setEditorWidgetCreator([]() { return new CppEditorWidget; });
+        setEditorCreator([]() { return new CppEditor; });
+        setAutoCompleterCreator([]() { return new CppAutoCompleter; });
+        setCommentStyle(CommentDefinition::CppStyle);
+        setCodeFoldingSupported(true);
+        setMarksVisible(true);
+        setParenthesesMatchingEnabled(true);
 
-    if (!Utils::HostOsInfo::isMacHost() && !Utils::HostOsInfo::isWindowsHost()) {
-        FileIconProvider::registerIconOverlayForMimeType(":/cppeditor/images/qt_cpp.png", CppEditor::Constants::CPP_SOURCE_MIMETYPE);
-        FileIconProvider::registerIconOverlayForMimeType(":/cppeditor/images/qt_c.png", CppEditor::Constants::C_SOURCE_MIMETYPE);
-        FileIconProvider::registerIconOverlayForMimeType(":/cppeditor/images/qt_h.png", CppEditor::Constants::CPP_HEADER_MIMETYPE);
+        setEditorActionHandlers(TextEditorActionHandler::Format
+                              | TextEditorActionHandler::UnCommentSelection
+                              | TextEditorActionHandler::UnCollapseAll
+                              | TextEditorActionHandler::FollowSymbolUnderCursor);
+
+        addHoverHandler(new CppHoverHandler);
+
+        if (!HostOsInfo::isMacHost() && !HostOsInfo::isWindowsHost()) {
+            FileIconProvider::registerIconOverlayForMimeType(
+                        QIcon(creatorTheme()->imageFile(Theme::IconOverlayCppSource, QLatin1String(":/cppeditor/images/qt_cpp.png"))),
+                        Constants::CPP_SOURCE_MIMETYPE);
+            FileIconProvider::registerIconOverlayForMimeType(
+                        QIcon(creatorTheme()->imageFile(Theme::IconOverlayCSource, QLatin1String(":/cppeditor/images/qt_c.png"))),
+                        Constants::C_SOURCE_MIMETYPE);
+            FileIconProvider::registerIconOverlayForMimeType(
+                        QIcon(creatorTheme()->imageFile(Theme::IconOverlayCppHeader, QLatin1String(":/cppeditor/images/qt_h.png"))),
+                        Constants::CPP_HEADER_MIMETYPE);
+        }
     }
-}
-
-IEditor *CppEditorFactory::createEditor()
-{
-    CPPEditorWidget *editor = new CPPEditorWidget();
-    editor->setRevisionsVisible(true);
-    m_owner->initializeEditor(editor);
-    return editor->editor();
-}
+};
 
 ///////////////////////////////// CppEditorPlugin //////////////////////////////////
 
 CppEditorPlugin *CppEditorPlugin::m_instance = 0;
 
 CppEditorPlugin::CppEditorPlugin() :
-    m_sortedOutline(true),
     m_renameSymbolUnderCursorAction(0),
     m_findUsagesAction(0),
     m_reparseExternallyChangedFiles(0),
@@ -130,27 +149,6 @@ CppEditorPlugin *CppEditorPlugin::instance()
     return m_instance;
 }
 
-void CppEditorPlugin::initializeEditor(CPPEditorWidget *editor)
-{
-    editor->setLanguageSettingsId(CppTools::Constants::CPP_SETTINGS_ID);
-    TextEditor::TextEditorSettings::initializeEditor(editor);
-
-    // function combo box sorting
-    connect(this, SIGNAL(outlineSortingChanged(bool)),
-            editor->outline(), SLOT(setSorted(bool)));
-}
-
-void CppEditorPlugin::setSortedOutline(bool sorted)
-{
-    m_sortedOutline = sorted;
-    emit outlineSortingChanged(sorted);
-}
-
-bool CppEditorPlugin::sortedOutline() const
-{
-    return m_sortedOutline;
-}
-
 CppQuickFixAssistProvider *CppEditorPlugin::quickFixProvider() const
 {
     return m_quickFixProvider;
@@ -158,62 +156,22 @@ CppQuickFixAssistProvider *CppEditorPlugin::quickFixProvider() const
 
 bool CppEditorPlugin::initialize(const QStringList & /*arguments*/, QString *errorMessage)
 {
-    if (!Core::MimeDatabase::addMimeTypes(QLatin1String(":/cppeditor/CppEditor.mimetypes.xml"), errorMessage))
-        return false;
+    Q_UNUSED(errorMessage)
+    Utils::MimeDatabase::addMimeTypes(QLatin1String(":/cppeditor/CppEditor.mimetypes.xml"));
 
-    addAutoReleasedObject(new CppEditorFactory(this));
-    addAutoReleasedObject(new CppHoverHandler);
+    addAutoReleasedObject(new CppEditorFactory);
     addAutoReleasedObject(new CppOutlineWidgetFactory);
     addAutoReleasedObject(new CppTypeHierarchyFactory);
     addAutoReleasedObject(new CppIncludeHierarchyFactory);
     addAutoReleasedObject(new CppSnippetProvider);
 
-    auto hf = new TextEditor::HighlighterFactory;
-    hf->setProductType<CppHighlighter>();
-    hf->setId(CppEditor::Constants::CPPEDITOR_ID);
-    hf->addMimeType(CppEditor::Constants::C_SOURCE_MIMETYPE);
-    hf->addMimeType(CppEditor::Constants::C_HEADER_MIMETYPE);
-    hf->addMimeType(CppEditor::Constants::CPP_SOURCE_MIMETYPE);
-    hf->addMimeType(CppEditor::Constants::CPP_HEADER_MIMETYPE);
-    addAutoReleasedObject(hf);
-
     m_quickFixProvider = new CppQuickFixAssistProvider;
     addAutoReleasedObject(m_quickFixProvider);
-    CppEditor::Internal::registerQuickFixes(this);
+    registerQuickFixes(this);
 
-    QString trCat = QCoreApplication::translate(Constants::WIZARD_CATEGORY, Constants::WIZARD_TR_CATEGORY);
+    Context context(Constants::CPPEDITOR_ID);
 
-    IWizardFactory *wizard = new CppClassWizard;
-    wizard->setWizardKind(IWizardFactory::ClassWizard);
-    wizard->setCategory(QLatin1String(Constants::WIZARD_CATEGORY));
-    wizard->setDisplayCategory(trCat);
-    wizard->setDisplayName(tr("C++ Class"));
-    wizard->setId(QLatin1String("A.Class"));
-    wizard->setDescription(tr("Creates a C++ header and a source file for a new class that you can add to a C++ project."));
-    addAutoReleasedObject(wizard);
-
-    wizard = new CppFileWizard(Source);
-    wizard->setWizardKind(IWizardFactory::FileWizard);
-    wizard->setCategory(QLatin1String(Constants::WIZARD_CATEGORY));
-    wizard->setDisplayCategory(trCat);
-    wizard->setDisplayName(tr("C++ Class"));
-    wizard->setDescription(tr("Creates a C++ source file that you can add to a C++ project."));
-    wizard->setDisplayName(tr("C++ Source File"));
-    wizard->setId(QLatin1String("B.Source"));
-    addAutoReleasedObject(wizard);
-
-    wizard = new CppFileWizard(Header);
-    wizard->setWizardKind(IWizardFactory::FileWizard);
-    wizard->setCategory(QLatin1String(Constants::WIZARD_CATEGORY));
-    wizard->setDisplayCategory(trCat);
-    wizard->setDescription(tr("Creates a C++ header file that you can add to a C++ project."));
-    wizard->setDisplayName(tr("C++ Header File"));
-    wizard->setId(QLatin1String("C.Header"));
-    addAutoReleasedObject(wizard);
-
-    Context context(CppEditor::Constants::C_CPPEDITOR);
-
-    ActionContainer *contextMenu = ActionManager::createMenu(CppEditor::Constants::M_CONTEXT);
+    ActionContainer *contextMenu = ActionManager::createMenu(Constants::M_CONTEXT);
 
     Command *cmd;
     ActionContainer *cppToolsMenu = ActionManager::actionContainer(CppTools::Constants::M_TOOLS_CPP);
@@ -248,7 +206,7 @@ bool CppEditorPlugin::initialize(const QStringList & /*arguments*/, QString *err
             new QAction(tr("Open Function Declaration/Definition in Next Split"), this);
     cmd = ActionManager::registerAction(openDeclarationDefinitionInNextSplit,
         Constants::OPEN_DECLARATION_DEFINITION_IN_NEXT_SPLIT, context, true);
-    cmd->setDefaultKeySequence(QKeySequence(Utils::HostOsInfo::isMacHost()
+    cmd->setDefaultKeySequence(QKeySequence(HostOsInfo::isMacHost()
                                             ? tr("Meta+E, Shift+F2")
                                             : tr("Ctrl+E, Shift+F2")));
     connect(openDeclarationDefinitionInNextSplit, SIGNAL(triggered()),
@@ -270,17 +228,16 @@ bool CppEditorPlugin::initialize(const QStringList & /*arguments*/, QString *err
     cppToolsMenu->addAction(cmd);
 
     m_openIncludeHierarchyAction = new QAction(tr("Open Include Hierarchy"), this);
-    cmd = Core::ActionManager::registerAction(m_openIncludeHierarchyAction, Constants::OPEN_INCLUDE_HIERARCHY, context);
-    cmd->setDefaultKeySequence(QKeySequence(Core::UseMacShortcuts ? tr("Meta+Shift+I") : tr("Ctrl+Shift+I")));
+    cmd = ActionManager::registerAction(m_openIncludeHierarchyAction, Constants::OPEN_INCLUDE_HIERARCHY, context);
+    cmd->setDefaultKeySequence(QKeySequence(UseMacShortcuts ? tr("Meta+Shift+I") : tr("Ctrl+Shift+I")));
     connect(m_openIncludeHierarchyAction, SIGNAL(triggered()), this, SLOT(openIncludeHierarchy()));
     contextMenu->addAction(cmd);
     cppToolsMenu->addAction(cmd);
 
     // Refactoring sub-menu
-    Context globalContext(Core::Constants::C_GLOBAL);
-    Command *sep = contextMenu->addSeparator(globalContext);
+    Command *sep = contextMenu->addSeparator();
     sep->action()->setObjectName(QLatin1String(Constants::M_REFACTORING_MENU_INSERTION_POINT));
-    contextMenu->addSeparator(globalContext);
+    contextMenu->addSeparator();
 
     m_renameSymbolUnderCursorAction = new QAction(tr("Rename Symbol Under Cursor"),
                                                   this);
@@ -293,17 +250,17 @@ bool CppEditorPlugin::initialize(const QStringList & /*arguments*/, QString *err
     cppToolsMenu->addAction(cmd);
 
     // Update context in global context
-    cppToolsMenu->addSeparator(globalContext);
+    cppToolsMenu->addSeparator();
     m_reparseExternallyChangedFiles = new QAction(tr("Reparse Externally Changed Files"), this);
-    cmd = ActionManager::registerAction(m_reparseExternallyChangedFiles, Constants::UPDATE_CODEMODEL, globalContext);
-    CppTools::CppModelManagerInterface *cppModelManager = CppTools::CppModelManagerInterface::instance();
+    cmd = ActionManager::registerAction(m_reparseExternallyChangedFiles, Constants::UPDATE_CODEMODEL);
+    CppTools::CppModelManager *cppModelManager = CppTools::CppModelManager::instance();
     connect(m_reparseExternallyChangedFiles, SIGNAL(triggered()), cppModelManager, SLOT(updateModifiedSourceFiles()));
     cppToolsMenu->addAction(cmd);
 
-    cppToolsMenu->addSeparator(globalContext);
+    cppToolsMenu->addSeparator();
     QAction *inspectCppCodeModel = new QAction(tr("Inspect C++ Code Model..."), this);
-    cmd = ActionManager::registerAction(inspectCppCodeModel, Constants::INSPECT_CPP_CODEMODEL, globalContext);
-    cmd->setDefaultKeySequence(QKeySequence(Core::UseMacShortcuts ? tr("Meta+Shift+F12") : tr("Ctrl+Shift+F12")));
+    cmd = ActionManager::registerAction(inspectCppCodeModel, Constants::INSPECT_CPP_CODEMODEL);
+    cmd->setDefaultKeySequence(QKeySequence(UseMacShortcuts ? tr("Meta+Shift+F12") : tr("Ctrl+Shift+F12")));
     connect(inspectCppCodeModel, SIGNAL(triggered()), this, SLOT(inspectCppCodeModel()));
     cppToolsMenu->addAction(cmd);
 
@@ -320,66 +277,54 @@ bool CppEditorPlugin::initialize(const QStringList & /*arguments*/, QString *err
     connect(ProgressManager::instance(), SIGNAL(allTasksFinished(Core::Id)),
             this, SLOT(onAllTasksFinished(Core::Id)));
 
-    readSettings();
     return true;
 }
 
-void CppEditorPlugin::readSettings()
-{
-    m_sortedOutline = ICore::settings()->value(QLatin1String("CppTools/SortedMethodOverview"), true).toBool();
-}
-
-void CppEditorPlugin::writeSettings()
-{
-    ICore::settings()->setValue(QLatin1String("CppTools/SortedMethodOverview"), m_sortedOutline);
-}
-
 void CppEditorPlugin::extensionsInitialized()
 {
 }
 
 ExtensionSystem::IPlugin::ShutdownFlag CppEditorPlugin::aboutToShutdown()
 {
-    writeSettings();
     return SynchronousShutdown;
 }
 
-static CPPEditorWidget *currentCppEditorWidget()
+static CppEditorWidget *currentCppEditorWidget()
 {
-    return qobject_cast<CPPEditorWidget*>(EditorManager::currentEditor()->widget());
+    return qobject_cast<CppEditorWidget*>(EditorManager::currentEditor()->widget());
 }
 
 void CppEditorPlugin::switchDeclarationDefinition()
 {
-    if (CPPEditorWidget *editorWidget = currentCppEditorWidget())
+    if (CppEditorWidget *editorWidget = currentCppEditorWidget())
         editorWidget->switchDeclarationDefinition(/*inNextSplit*/ false);
 }
 
 void CppEditorPlugin::openDeclarationDefinitionInNextSplit()
 {
-    if (CPPEditorWidget *editorWidget = currentCppEditorWidget())
+    if (CppEditorWidget *editorWidget = currentCppEditorWidget())
         editorWidget->switchDeclarationDefinition(/*inNextSplit*/ true);
 }
 
 void CppEditorPlugin::renameSymbolUnderCursor()
 {
-    if (CPPEditorWidget *editorWidget = currentCppEditorWidget())
+    if (CppEditorWidget *editorWidget = currentCppEditorWidget())
         editorWidget->renameSymbolUnderCursor();
 }
 
 void CppEditorPlugin::findUsages()
 {
-    if (CPPEditorWidget *editorWidget = currentCppEditorWidget())
+    if (CppEditorWidget *editorWidget = currentCppEditorWidget())
         editorWidget->findUsages();
 }
 
 void CppEditorPlugin::showPreProcessorDialog()
 {
-    if (CPPEditorWidget *editorWidget = currentCppEditorWidget())
+    if (CppEditorWidget *editorWidget = currentCppEditorWidget())
         editorWidget->showPreProcessorWidget();
 }
 
-void CppEditorPlugin::onTaskStarted(Core::Id type)
+void CppEditorPlugin::onTaskStarted(Id type)
 {
     if (type == CppTools::Constants::TASK_INDEX) {
         m_renameSymbolUnderCursorAction->setEnabled(false);
@@ -390,7 +335,7 @@ void CppEditorPlugin::onTaskStarted(Core::Id type)
     }
 }
 
-void CppEditorPlugin::onAllTasksFinished(Core::Id type)
+void CppEditorPlugin::onAllTasksFinished(Id type)
 {
     if (type == CppTools::Constants::TASK_INDEX) {
         m_renameSymbolUnderCursorAction->setEnabled(true);
@@ -411,6 +356,15 @@ void CppEditorPlugin::inspectCppCodeModel()
     }
 }
 
+#ifdef WITH_TESTS
+QList<QObject *> CppEditorPlugin::createTestObjects() const
+{
+    return QList<QObject *>()
+        << new Tests::DoxygenTest
+        ;
+}
+#endif
+
 void CppEditorPlugin::openTypeHierarchy()
 {
     if (currentCppEditorWidget()) {
@@ -423,10 +377,11 @@ void CppEditorPlugin::openTypeHierarchy()
 void CppEditorPlugin::openIncludeHierarchy()
 {
     if (currentCppEditorWidget()) {
-        Core::NavigationWidget *navigation = Core::NavigationWidget::instance();
-        navigation->activateSubWidget(Core::Id(Constants::INCLUDE_HIERARCHY_ID));
+        NavigationWidget *navigation = NavigationWidget::instance();
+        navigation->activateSubWidget(Id(Constants::INCLUDE_HIERARCHY_ID));
         emit includeHierarchyRequested();
     }
 }
 
-Q_EXPORT_PLUGIN(CppEditorPlugin)
+} // namespace Internal
+} // namespace CppEditor
